from kart import miners, mappers, renderers
from http.server import SimpleHTTPRequestHandler
from socketserver import TCPServer
import os
import shutil
import argparse


class Kart:
    def __init__(self):
        self.miners = [
            miners.DefaultCollectionMiner(),
            miners.DefaultDataMiner(),
            miners.DefaultPageMiner(),
        ]
        self.mappers = [mappers.ManualMapper()]
        self.renderers = [renderers.JinjaSiteRenderer()]

    def build(self, build_location="_site"):

        self.build_location = build_location

        self.site = {}
        for miner in self.miners:
            self.site.update(miner.collect())

        # print(self.site)

        self.map = []
        for mapper in self.mappers:
            self.map.extend(mapper.map(self.site))
        self.url_map = {}
        for page in self.map:
            self.url_map.update({page["name"]: page["url"]})

        # print(self.map)
        # print(self.url_map)

        os.makedirs(build_location, exist_ok=True)
        for x in os.listdir(build_location):
            if os.path.isdir(os.path.join(build_location, x)):
                shutil.rmtree(os.path.join(build_location, x))
            else:
                os.remove(os.path.join(build_location, x))
        shutil.copytree("static", os.path.join(build_location, "static"))
        for renderer in self.renderers:
            renderer.render(
                self.map,
                self.site,
                build_location=build_location,
                url_function=self.url,
            )

        # print(self.map)

    def url(self, name):
        try:
            result = self.url_map[name]
        except:
            result = self.url_map[name + ".1"]
        return result

    def serve(self, port=9000):
        os.chdir(self.build_location)
        server_address = ("", port)
        with TCPServer(server_address, SimpleHTTPRequestHandler) as httpd:
            httpd.serve_forever()

    def run(self):
        parser = argparse.ArgumentParser()
        parser.add_argument(
            "command", help="command to execute", choices={"build", "serve"}
        )
        args = parser.parse_args()
        if args.command == "build":
            self.build()
        if args.command == "serve":
            self.build()
            self.serve()



import os
import frontmatter
import yaml
import markdown


class Miner:
    # def __init__(self, model, location):
    #     self.model = model
    #     self.location = location
    #     self.data = {}
    #
    # def parse(self):
    #     pass
    pass


class DefaultCollectionMiner(Miner):
    def __init__(self, model=None, location="collections"):
        self.model = model
        self.location = location
        self.data = {}

    def collect(self):
        for dir in os.listdir("collections"):
            self.data[dir] = []
            for file in os.listdir(os.path.join(self.location, dir)):
                with open(os.path.join(self.location, dir, file), "r") as f:
                    metadata, content = frontmatter.parse(f.read())
                    object = metadata
                    object["slug"] = file.split(".")[0]
                    object["content"] = markdown.markdown(content)
                    self.data[dir].append(object)
        return {"collections": self.data}


class DefaultPageMiner(Miner):
    def __init__(self, location="pages"):
        self.location = location
        self.data = {}

    def collect(self):
        for file in os.listdir(self.location):
            with open(os.path.join(self.location, file), "r") as f:
                metadata, content = frontmatter.parse(f.read())
                object = metadata
                object["content"] = markdown.markdown(content)
                self.data[file.split(".")[0]] = object
        return {"pages": self.data}


class DefaultDataMiner(Miner):
    def __init__(self, location="data"):
        self.location = location
        self.data = {}

    def collect(self):
        for file in os.listdir(self.location):
            with open(os.path.join(self.location, file), "r") as f:
                self.data[file.split(".")[0]] = yaml.full_load(f.read())
        return {"data": self.data}



def url(name, object):
    pass


class AutomaticMapper:
    def __init__(self):
        self.urls = []

    def map(self, site):
        return {}


class ManualMapper:
    def __init__(self):
        self.rules = []  # a rule is a function
        self.urls = []

    def map(self, site):

        for rule in self.rules:
            self.urls.extend(rule(site))

        return self.urls



from jinja2 import Environment, FileSystemLoader
import os


class JinjaSiteRenderer:
    def __init__(self, name="main_renderer", template_folder="templates"):
        self.name = name
        self.template_folder = template_folder

    def render(self, map, site, url_function, build_location="_site"):
        env = Environment(loader=FileSystemLoader(self.template_folder))
        for page in map:
            if self.name not in page["renderers"]:
                continue
            template = ""
            if "template" in page["data"].keys():
                template = page["data"]["template"]
            if not template:
                template = page["default_template"]
            jinja_template = env.get_template(template)
            rendered_file = jinja_template.render(
                page=page["data"], site=site, url=url_function
            )
            os.makedirs(build_location + page["url"], exist_ok=True)
            with open(build_location + page["url"] + "index.html", "w") as f:
                f.write(rendered_file)



import setuptools


with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="Kart",
    version="0.1.0",
    author="Giacomo Caironi",
    author_email="giacomo.caironi@gmail.com",
    description="A very flexible static site generator written in python",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/giacomocaironi/Kart",
    packages=setuptools.find_packages(),
    install_requires=["jinja2", "pyyaml", "markdown", "python-frontmatter"],
)
